@@ -17,7 +17,7 @@
 
 package Git::Hooks::CheckStructure;
 {
-  $Git::Hooks::CheckStructure::VERSION = '0.024';
+  $Git::Hooks::CheckStructure::VERSION = '0.025';
 }
 # ABSTRACT: Git::Hooks plugin for ref/file structure validation.
 
@@ -32,27 +32,19 @@ use Error qw(:try);
 
 (my $HOOK = __PACKAGE__) =~ s/.*:://;
 
-#############
-# Grok hook configuration and set defaults.
-
-my $Config = hook_config($HOOK);
-
 ##########
 
-sub file_structure {
-    return unless exists $Config->{file};
-    local $@ = undef;
-    state $structure = eval { eval_gitconfig($Config->{file}[-1]) };
-    die "$HOOK: $@\n" if $@;
-    return $structure;
-}
+sub get_structure {
+    my ($git, $what) = @_;
 
-sub ref_structure {
-    return unless exists $Config->{ref};
-    local $@ = undef;
-    state $structure = eval { eval_gitconfig($Config->{ref}[-1]) };
-    die "$HOOK: $@\n" if $@;
-    return $structure;
+    if (my $value = $git->config($HOOK => $what)) {
+        local $@ = undef;
+        my $structure = eval {eval_gitconfig($value)};
+        die "$HOOK: $@\n" if $@;
+        return $structure;
+    } else {
+        return;
+    }
 }
 
 sub check_array_structure {
@@ -128,13 +120,13 @@ sub check_structure {
 }
 
 sub check_added_files {
-    my ($files) = @_;
+    my ($git, $files) = @_;
     my @errors;
     foreach my $file (sort keys %$files) {
         # Split the $file path in its components. We prefix $file with
         # a slash to make it look like an absolute path for
         # check_structure.
-        my ($code, $error) = check_structure(file_structure(), [split '/', "/$file"]);
+        my ($code, $error) = check_structure(get_structure($git, 'file'), [split '/', "/$file"]);
         push @errors, "$error: $file" if $code == 0;
     }
     return @errors;
@@ -145,10 +137,10 @@ sub check_ref {
 
     my @errors;
 
-    my ($old_commit, $new_commit) = get_affected_ref_range($ref);
+    my ($old_commit, $new_commit) = $git->get_affected_ref_range($ref);
 
     # Check names of newly created refs
-    if (my $structure = ref_structure()) {
+    if (my $structure = get_structure($git, 'ref')) {
         if ($old_commit eq '0' x 40) {
             check_structure($structure, [split '/', "/$ref"])
                 or push @errors, "reference name '$ref' not allowed";
@@ -156,8 +148,8 @@ sub check_ref {
     }
 
     # Check names of newly added files
-    if (file_structure()) {
-        push @errors, check_added_files($git->get_diff_files('--diff-filter=A', $old_commit, $new_commit));
+    if (get_structure($git, 'file')) {
+        push @errors, check_added_files($git, $git->get_diff_files('--diff-filter=A', $old_commit, $new_commit));
     }
 
     die join("\n", "$HOOK: errors in ref '$ref' commits", @errors), "\n" if @errors;
@@ -169,9 +161,9 @@ sub check_ref {
 sub check_affected_refs {
     my ($git) = @_;
 
-    return if im_admin();
+    return if im_admin($git);
 
-    foreach my $ref (get_affected_refs()) {
+    foreach my $ref ($git->get_affected_refs()) {
         check_ref($git, $ref);
     }
 
@@ -181,7 +173,7 @@ sub check_affected_refs {
 sub check_commit {
     my ($git) = @_;
 
-    my @errors = check_added_files($git->get_diff_files('--diff-filter=A', '--cached'));
+    my @errors = check_added_files($git, $git->get_diff_files('--diff-filter=A', '--cached'));
 
     die join("\n", "$HOOK: errors in commit", @errors), "\n" if @errors;
 
@@ -205,7 +197,7 @@ Git::Hooks::CheckStructure - Git::Hooks plugin for ref/file structure validation
 
 =head1 VERSION
 
-version 0.024
+version 0.025
 
 =head1 DESCRIPTION
 
@@ -237,7 +229,7 @@ comply with its structure definition.
 
 =back
 
-=for Pod::Coverage check_added_files check_ref file_structure ref_structure check_array_structure check_string_structure
+=for Pod::Coverage check_added_files check_ref get_structure check_array_structure check_string_structure
 
 =head1 NAME
 
